"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const immer = require("immer");
const R = require("../src/writer");
const fs = require("fs");
function genericComment(str) {
    return `/* ${str} */`;
}
class TestClass {
    constructor() {
        this.lines = [];
    }
}
const CreateIfNode = (condition, thenBlock, elseBlock) => {
    return [
        R.Join(["if(", condition, ") {"]),
        [[thenBlock]],
        elseBlock ? ["} else {", [[elseBlock]], "}"] : "}",
    ];
};
describe("Writer generator tests", () => {
    test("README sample 1", () => {
        const ctx = new R.Ctx(null);
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [
            ["if( x > 10){"],
            [["console.log('x was bigger than ten');"]],
            "} else {",
            [["console.log('x was smaller or equal to ten');"]],
            "}",
        ]);
        console.log(ctx.writer.getCode());
    });
    test("README sample 2", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [
            CreateIfNode("x > 10", "console.log('x was bigger than ten');", "console.log('x was smaller or equal to ten');"),
            CreateIfNode("x < 0", "console.log('x was less than zero');"),
        ]);
        console.log(ctx.writer.getCode());
    });
    test("Test line generation", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [["line1"], ["line2"]]);
        chai_1.expect(ctx.writer.getCode()).to.deep.eq(`line1\nline2\n`);
    });
    test("Test contextless generator", () => {
        const fs = new R.CodeFileSystem();
        const code = fs
            .getFile("./builder/gen", "testout.txt")
            .getWriter()
            .walk([["line1"], ["line2"]])
            .getCode();
        chai_1.expect(code).to.deep.eq(`line1\nline2\n`);
    });
    test("Test custom context with .walk", () => {
        const fs = new R.CodeFileSystem();
        const code = fs
            .getFile("./builder/gen", "testout.txt")
            .getWriter()
            .walk([
            (ctx) => {
                ctx.data = { str: "hello" };
            },
            ["line1"],
            ["line2"],
            (ctx) => ctx.data.str,
        ])
            .getCode();
        chai_1.expect(code).to.deep.eq(`line1\nline2\nhello\n`);
        const code2 = fs
            .getFile("./builder/gen", "testout2.txt")
            .getWriter()
            .walk([
            (ctx) => {
                ctx.data = "OK";
            },
            ["line1"],
            ["line2"],
            (ctx) => ctx.data,
        ])
            .getCode();
        chai_1.expect(code2).to.deep.eq(`line1\nline2\nOK\n`);
    });
    test("Test custom context with .walk with defined param", () => {
        const fs = new R.CodeFileSystem();
        // define a context with data
        const ctx = new R.Ctx();
        ctx.data = { str: "custom" };
        const code = fs
            .getFile("./builder/gen", "testout.txt")
            .getWriter()
            .walk([["line1"], ["line2"], (ctx) => ctx.data.str], ctx)
            .getCode();
        chai_1.expect(code).to.deep.eq(`line1\nline2\ncustom\n`);
    });
    test("generic comment creator", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [genericComment("ok")]);
        chai_1.expect(ctx.writer.getCode()).to.deep.eq(`/* ok */\n`);
    });
    test("Test block with indent", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [["if(){"], [["line2", "line3"]], "}"]);
        chai_1.expect(ctx.writer.getCode()).to.deep.eq(`if(){\n  line2\n  line3\n}\n`);
    });
    test("Test generator function", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [["if(){"], [[(ctx) => "line2", "line3"]], "}"]);
        chai_1.expect(ctx.writer.getCode()).to.deep.eq(`if(){\n  line2\n  line3\n}\n`);
    });
    test("Test join function", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [["if(){"], [[(ctx) => R.Join(["lin", "e2"]), "line3"]], "}"]);
        chai_1.expect(ctx.writer.getCode()).to.deep.eq(`if(){\n  line2\n  line3\n}\n`);
    });
    test("generic recursive generators", () => {
        const ctx = new R.Ctx();
        const fs = new R.CodeFileSystem();
        ctx.writer = fs.getFile("./builder/gen", "testout.txt").getWriter();
        R.Walk(ctx, [(ctx) => (ctx) => (ctx) => "ok"]);
        chai_1.expect(ctx.writer.getCode()).to.deep.eq("ok\n");
    });
    test("Test simple Context Creator", () => {
        chai_1.expect(R.Walk(R.CreateContext({ cnt: 1 }), [
            (ctx) => {
                ctx.data.cnt++;
            },
        ]).data.cnt).to.equal(2);
        // using immer to mutate state is quite fine
        chai_1.expect(R.Walk(R.CreateContext({ cnt: 1 }), [
            (ctx) => {
                ctx.data = immer.produce(ctx.data, (d) => {
                    d.cnt += 2;
                });
            },
        ]).data.cnt).to.equal(3);
        chai_1.expect(R.Walk(R.CreateContext({ cnt: 1 }), [
            (ctx) => {
                ctx.data = immer.produce(ctx.data, (d) => {
                    d.cnt += 2;
                });
            },
            (ctx) => {
                ctx.data = immer.produce(ctx.data, (d) => {
                    d.cnt += 2;
                });
            },
        ]).data.cnt).to.equal(5);
        chai_1.expect(R.Walk(R.CreateContext({ cnt: 1 }), [
            (ctx) => ctx.produce((d) => {
                d.cnt++;
            }),
            (ctx) => ctx.produce((d) => {
                d.cnt++;
            }),
        ]).data.cnt).to.equal(3);
        const ctx = R.CreateContext({ cnt: 1 });
        const myFns = [
            (ctx) => ctx.produce((d) => {
                d.cnt++;
            }),
            (ctx) => ctx.produce((d) => {
                d.cnt++;
            }),
        ];
        R.Walk(ctx, myFns);
        R.Walk(ctx, myFns);
        chai_1.expect(ctx.data.cnt).to.equal(5);
        chai_1.expect(R.Walk(R.CreateContext(new TestClass()), [
            (ctx) => {
                ctx.data.lines.push("Does");
            },
            (ctx) => {
                ctx.data.lines.push("This");
            },
            (ctx) => {
                ctx.data.lines.push("Work");
            },
        ]).data.lines.join(" ")).to.equal("Does This Work");
        chai_1.expect(R.Walk(R.CreateContext(new TestClass()), [
            (ctx) => ctx.produce((c) => {
                c.lines.push("Does");
            }),
            (ctx) => ctx.produce((c) => {
                c.lines.push("This");
            }),
            (ctx) => ctx.produce((c) => {
                c.lines.push("Work");
            }),
        ]).data.lines.join(" ")).to.equal("Does This Work");
        const DefineIntVariable = (name, value) => {
            return (ctx) => {
                ctx.produce((data) => {
                    data.definedVariables.push(name);
                    data.intVariables.push({
                        name,
                        value,
                    });
                });
            };
        };
        const IncrementVariable = (name, value) => {
            return (ctx) => {
                ctx.produce((data) => {
                    for (let i = 0; i < data.intVariables.length; i++) {
                        if (data.intVariables[i].name === name) {
                            data.prevCtx = ctx.data;
                            data.intVariables[i].value += value;
                        }
                    }
                });
            };
        };
        const vCtx = { intVariables: [], definedVariables: [] };
        const varCtx = R.Walk(R.CreateContext(vCtx), [
            DefineIntVariable("x", 10),
            DefineIntVariable("y", 200),
            IncrementVariable("x", 11),
            (ctx) => {
                const newCtx = ctx.fork();
                return () => {
                    R.Walk(newCtx, [
                        IncrementVariable("x", 1),
                        (ctx) => {
                            console.log("Sub ctx ", ctx.data);
                            chai_1.expect(ctx.data.intVariables[0].value).to.equal(22);
                            chai_1.expect(ctx.data.intVariables[1].value).to.equal(200);
                        },
                    ]);
                };
            },
            (ctx) => {
                const newCtx = ctx.fork();
                return () => {
                    R.Walk(newCtx, [
                        IncrementVariable("x", 101),
                        (ctx) => {
                            console.log("Sub ctx ", ctx.data);
                            chai_1.expect(ctx.data.intVariables[0].value).to.equal(122);
                        },
                    ]);
                };
            },
        ]);
        chai_1.expect(varCtx.data.intVariables[0].name).to.equal("x");
        chai_1.expect(varCtx.data.intVariables[0].value).to.equal(21);
        chai_1.expect(varCtx.data.prevCtx.intVariables[0].value).to.equal(10);
        console.log("prev", varCtx.data.prevCtx);
    });
    test("Walk Context in Steps", () => {
        const DefineIntVariable = (name, value) => {
            return (ctx) => {
                ctx.produce((data) => {
                    data.definedVariables.push(name);
                    data.intVariables.push({
                        name,
                        value,
                    });
                });
            };
        };
        const IncrementVariable = (name, value) => {
            return (ctx) => {
                ctx.produce((data) => {
                    for (let i = 0; i < data.intVariables.length; i++) {
                        if (data.intVariables[i].name === name) {
                            data.prevCtx = ctx.data;
                            data.intVariables[i].value += value;
                        }
                    }
                });
            };
        };
        const vCtx = { intVariables: [], definedVariables: [] };
        // defines something operated on the context
        const step = [IncrementVariable("x", 1)];
        const varCtx = R.Walk(R.CreateContext(vCtx), [
            DefineIntVariable("x", 10),
            DefineIntVariable("y", 200),
        ]);
        const ctx2 = R.Walk(varCtx, step);
        chai_1.expect(ctx2.data.intVariables[0].value).to.equal(11);
        const ctx3 = R.Walk(ctx2, step);
        chai_1.expect(ctx3.data.intVariables[0].value).to.equal(12);
        const ctx4 = R.Walk(varCtx, [DefineIntVariable("x2", 77)]);
        chai_1.expect(ctx4.data.intVariables[2].value).to.equal(77);
        console.log(R.Walk(ctx4, (ctx) => {
            return ctx.data.intVariables.map((varName) => `const ${varName.name}:number = ${varName.value};`);
        }).writer.getCode());
        /*
        const x:number = 12;
        const y:number = 200;
        const x2:number = 77;
        */
    });
    test("Example in the Docs", () => {
        const data = {
            values: ["value1", "value2", "value3"],
        };
        const ctx = R.CreateContext(data);
        const newCtx = R.Walk(ctx, (ctx) => [
            `switch( value ) {`,
            ...ctx.data.values.map((name) => [
                [
                    `case "${name}":`,
                    [[`console.log("Found value ${name}");`, "break;"]],
                ],
            ]),
            "}",
        ]);
        console.log(newCtx.writer.getCode());
    });
    test("Test input test", () => {
        const code1 = [
            'import * as fs from "fs";',
            "",
            "class Foo {",
            [["abc() {", [['console.log("ABC");', "return true"]], "}"]],
            "}",
            "",
        ];
        const code2 = [
            'import * as fs from "fs";',
            "",
            "class Foo {",
            [
                [
                    "anotherRoute() {",
                    [['return "This is good!";']],
                    "}",
                    "",
                    "abc() {",
                    [
                        [
                            'console.log("ABC");',
                            "for (let i = 0; i < 10; i++) {",
                            [['console.log(" i = ", i);']],
                            "}",
                            "return true",
                        ],
                    ],
                    "}",
                ],
            ],
            "}",
            "",
        ];
        function AreEqual(generated, current) {
            if (typeof generated === typeof current) {
                if (generated instanceof Array) {
                    return true;
                }
                return generated === current;
            }
            return false;
        }
        function GetValue(generated, current) {
            if (typeof generated === typeof current) {
                if (generated instanceof Array) {
                    return WalkCode(generated, current);
                }
                return generated;
            }
            return "";
        }
        function WalkCode(generated, current) {
            let ci = 0;
            let i = 0;
            const output = [];
            while (ci < current.length) {
                if (AreEqual(generated[i], current[ci])) {
                    output.push(GetValue(generated[i], current[ci]));
                    ci++;
                    i++;
                    continue;
                }
                output.push(current[ci]);
                ci++;
            }
            return output;
        }
        fs.writeFileSync("./test/filegenerator/merged.json", JSON.stringify(WalkCode(code1, code2), null, 2));
        const input1 = R.TextToArray(fs.readFileSync("test/input/testinput.ts", "utf8"));
        const input2 = R.TextToArray(fs.readFileSync("test/input/testinput_changed.ts", "utf8"));
        fs.writeFileSync("./test/filegenerator/input1.json", JSON.stringify(input1, null, 2));
        fs.writeFileSync("./test/filegenerator/input2.json", JSON.stringify(input2, null, 2));
        R.CreateContext({})
            .file("./", "diff_test2.ts")
            .write(input2)
            .save("./test/difftest", { useDiff: true, usePrettier: true });
        R.CreateContext({})
            .file("./", "diff_test.ts")
            .write([`if(true) {`, [[`console.log("OK");`]], `}`])
            .save("./test/difftest", { useDiff: true, usePrettier: true });
    });
    test("File generator test", () => {
        R.CreateContext({})
            .file("./", "writer.ts")
            .write([
            "const fileWriterGenerator = () => ",
            R.TextGenerator(fs.readFileSync("src/writer/index.ts", "utf8")),
        ])
            .save("./test/filegenerator");
        R.CreateContext({})
            .file("./", "fancywriter.ts")
            .write([
            "const fancyFileWriterGenerator = () => ",
            R.TextGenerator(fs.readFileSync("src/writer/index.ts", "utf8"), (s) => s.replace("CodeBlock", "FancyCodeBlock")),
        ])
            .save("./test/filegenerator");
        R.CreateContext({})
            .file("./", "output_example.ts")
            .write([
            "const outputExample = () => ",
            R.TextGenerator(fs.readFileSync("test/output/example.ts", "utf8")),
        ])
            .save("./test/filegenerator");
        R.CreateContext({})
            .file("./", "test_yml.ts")
            .write([
            "const outputExampleForYml = () => ",
            R.TextGenerator(fs.readFileSync("test/test.yml", "utf8")),
        ])
            .save("./test/filegenerator");
    });
});
//# sourceMappingURL=filesystem.test.js.map